import(survival)
import(glmnet)
import(hdnom)
import(MASS)
#import(stats)
import(mice)

importFrom("utils", "txtProgressBar", "setTxtProgressBar")
importFrom("graphics", "abline", "lines", "plot.default", "grid", "segments", "legend", "points")
importFrom("grDevices", "adjustcolor", "rainbow")
importFrom("methods", "hasArg")
importFrom("graphics", "axis")
importFrom("stats", "as.formula", "formula", "glm", "loess", "median",
             "model.matrix", "na.omit", "pnorm", "predict",
             "printCoefmat", "qnorm", "quantile", "sd", "terms",
			       "coef","plogis","vcov", "update")

export(gc_binary)
export(gc_continuous)
export(gc_times)
export(gc_count)
export(transport)

S3method(print, gctimes)
S3method(plot, gctimes)
S3method(summary, gctimes)
S3method(print, gcbinary)
S3method(plot, gcbinary)
S3method(summary, gcbinary)
S3method(print, gccontinuous)
S3method(plot, gccontinuous)
S3method(summary, gccontinuous)
S3method(print, gccount)
S3method(plot, gccount)
S3method(summary, gccount)
S3method(print, summary.gctimes)
S3method(print, summary.gcbinary)
S3method(print, summary.gccount)
S3method(print, summary.gccontinuous)
